#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define N 16
#define LN 4
#define PI 3.14159265358979323846

typedef struct { double re; double im; } complex;

complex *omega, *aux0, *aux1, *aux2;
unsigned int lsize, size, **ffte;

void
init_ffte()
{
  extern unsigned int lsize, size, **ffte;
  unsigned int i, j, k;
  
  size = 1<<lsize;

  ffte = (unsigned int **)(calloc(lsize,sizeof(int *)));
  for ( i = 0 ; i < lsize ; i++ )
  {
    ffte[i] = (unsigned int *)(calloc(size,sizeof(int)));
    for ( j = 0 ; j < size ; j++ )
    {
      ffte[i][j] = 0;
      for ( k = 0 ; k <= i ; k++ )
        if ( j & (1<<k) ) ffte[i][j] |= 1<<(lsize-1-k);
    }
  }
}

void
init_omega()
{
  extern unsigned int size;
  unsigned int i;

  omega = (complex *)(calloc(size,sizeof(complex)));
  aux0 = (complex *)(calloc(size,sizeof(complex)));
  aux1 = (complex *)(calloc(size,sizeof(complex)));
  aux2 = (complex *)(calloc(size,sizeof(complex)));
  for ( i = 0 ; i < size ; i++ )
  { omega[i].re = cos((PI*2*i)/size); omega[i].im = sin((PI*2*i)/size); }
}

void
fft_step(unsigned int step, complex *pin, complex *pout)
{
  extern unsigned int size, lsize, **ffte;
  extern complex *omega;

  unsigned int i, i0, i1, e;
  
  for ( i = 0 ; i < size ; i++ )
  {
    i0 = i>>1;
    i1 = i0 | 1<<(lsize-1);
    e = ffte[step][i];
    pout[i].re = pin[i0].re + (omega[e].re * pin[i1].re) - (omega[e].im * pin[i1].im);
    pout[i].im = pin[i0].im + (omega[e].re * pin[i1].im) + (omega[e].im * pin[i1].re);
  }
}

void
fft(complex *pin, complex *pout)
{
  extern unsigned int lsize;
  extern complex *aux0, *aux1;
  unsigned int i;

  fft_step(0,pin,aux0);

  for ( i = 1 ; i < lsize  ; i++ )
  { if ( i & 1 ) { fft_step(i,aux0,aux1); } else { fft_step(i,aux1,aux0); } }

  if ( lsize & 1 )
  {
    for ( i = 0 ; i < size ; i++ )
    {
      pout[i].re = aux0[ffte[lsize-1][i]].re;
      pout[i].im = aux0[ffte[lsize-1][i]].im;
    }
  }
  else
  {
    for ( i = 0 ; i < size ; i++ )
    {
      pout[i].re = aux1[ffte[lsize-1][i]].re;
      pout[i].im = aux1[ffte[lsize-1][i]].im;
    }
  }
}

void
ffti(complex *pin, complex *pout)
{
  extern unsigned int size;
  extern complex *aux2;
  double is;
  unsigned int i;

  is = 1/((double)(size));
  fft(pin,aux2);

  pout[0].re = is * aux2[0].re;
  pout[0].im = is * aux2[0].im;
  for ( i = 1 ; i < size ; i++ )
  { 
    pout[i].re = is * aux2[size-i].re;
    pout[i].im = is * aux2[size-i].im;
  }
}

int
main(int argc, char *argv[])
{
  extern unsigned int lsize, size;
  unsigned int i;
  complex *a, *b;

  lsize = atoi(argv[1]);
  size = 1<<(lsize);
  a = (complex *)(calloc(size,sizeof(complex)));
  b = (complex *)(calloc(size,sizeof(complex)));

  init_ffte();
  init_omega();

  for ( i = 0 ; i < size ; i++ ) scanf("%lf %lf",&(a[i].re),&(a[i].im));

  fft(a,b);

  for ( i = 0 ; i < size ; i++ ) printf("%f\t%f\n",b[i].re,b[i].im);

  ffti(b,a);

  for ( i = 0 ; i < size ; i++ ) printf("%f\t%f\n",a[i].re,a[i].im);
/***********************************************************************
  for ( i = 0 ; i < lsize ; i++ ) {
  for ( j = 0 ; j < size ; j++ ) printf("%d\t",ffte[i][j]);
  printf("\n"); }

  printf("\n");
  for ( i = 0 ; i < size ; i++ ) printf("%f\t%f\n",omega[i].re,omega[i].im);
***********************************************************************/

  return(0);
}
